@@ -1,4 +1,4 @@
-/* $Header: /u/christos/src/tcsh-6.05/RCS/ed.inputl.c,v 3.39 1994/05/07 18:51:25 christos Exp $ */
+/* $Header: /u/christos/src/tcsh-6.06/RCS/ed.inputl.c,v 3.43 1995/04/16 19:15:53 christos Exp $ */
 /*
  * ed.inputl.c: Input line handling.
  */
@@ -36,7 +36,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: ed.inputl.c,v 3.39 1994/05/07 18:51:25 christos Exp $")
+RCSID("$Id: ed.inputl.c,v 3.43 1995/04/16 19:15:53 christos Exp $")
 
 #include "ed.h"
 #include "ed.defns.h"		/* for the function names */
@@ -90,7 +90,7 @@ Inputl()
     Char    ch;
     int     num;		/* how many chars we have read at NL */
     int	    expnum;
-    struct varent *crct = adrof(STRcorrect);
+    struct varent *crct = inheredoc ? NULL : adrof(STRcorrect);
     struct varent *autol = adrof(STRautolist);
     struct varent *matchbeep = adrof(STRmatchbeep);
     struct varent *imode = adrof(STRinputmode);
@@ -178,7 +178,7 @@ Inputl()
 
 	if (cmdnum >= NumFuns) {/* BUG CHECK command */
 #ifdef DEBUG_EDIT
-	    xprintf("ERROR: illegal command from key 0%o\r\n", ch);
+	    xprintf(CGETS(6, 1, "ERROR: illegal command from key 0%o\r\n"), ch);
 #endif
 	    continue;		/* try again */
 	}
@@ -189,6 +189,9 @@ Inputl()
 	/* save the last command here */
 	LastCmd = cmdnum;
 
+	/* make sure fn is initialized */
+	fn = (retval == CC_COMPLETE_ALL) ? LIST_ALL : LIST;
+
 	/* use any return value */
 	switch (retval) {
 
@@ -244,13 +247,13 @@ Inputl()
 		    ch = tch;
 		    if (ch == 'y' || ch == ' ') {
 			LastChar = CorrChar;	/* Restore the corrected end */
-			xprintf("yes\n");
+			xprintf(CGETS(6, 2, "yes\n"));
 		    }
 		    else {
 			copyn(InputBuf, Origin, INBUFSIZE);
 			LastChar = SaveChar;
 			if (ch == 'e') {
-			    xprintf("edit\n");
+			    xprintf(CGETS(6, 3, "edit\n"));
 			    *LastChar-- = '\0';
 			    Cursor = LastChar;
 			    printprompt(3, NULL);
@@ -260,14 +263,14 @@ Inputl()
 			    break;
 			}
 			else if (ch == 'a') {
-			    xprintf("abort\n");
+			    xprintf(CGETS(6, 4, "abort\n"));
 		            LastChar = InputBuf;   /* Null the current line */
 			    Cursor = LastChar;
 			    printprompt(0, NULL);
 			    Refresh();
 			    break;
 			}
-			xprintf("no\n");
+			xprintf(CGETS(6, 5, "no\n"));
 		    }
 		    flush();
 		}
@@ -285,9 +288,9 @@ Inputl()
                     PastBottom();
 		}
 		if (matchval == 0) {
-		    xprintf("No matching command\n");
+		    xprintf(CGETS(6, 6, "No matching command\n"));
 		} else if (matchval == 2) {
-		    xprintf("Ambiguous command\n");
+		    xprintf(CGETS(6, 7, "Ambiguous command\n"));
 		}
 	        if (NeedsRedraw) {
 		    ClearLines();
@@ -504,7 +507,7 @@ Inputl()
 
 	case CC_FATAL:		/* fatal error, reset to known state */
 #ifdef DEBUG_EDIT
-	    xprintf("*** editor fatal ERROR ***\r\n\n");
+	    xprintf(CGETS(7, 8, "*** editor fatal ERROR ***\r\n\n"));
 #endif				/* DEBUG_EDIT */
 	    /* put (real) cursor in a known place */
 	    ClearDisp();	/* reset the display stuff */
@@ -662,15 +665,18 @@ GetNextCommand(cmdnum, ch)
 	cmd = CurrentKeyMap[(unsigned char) *ch];
 	if (cmd == F_XKEY) {
 	    XmapVal val;
-	    switch (GetXkey(ch, &val)) {
+	    CStr cstr;
+	    cstr.buf = ch;
+	    cstr.len = Strlen(ch);
+	    switch (GetXkey(&cstr, &val)) {
 	    case XK_CMD:
 		cmd = val.cmd;
 		break;
 	    case XK_STR:
-		PushMacro(val.str);
+		PushMacro(val.str.buf);
 		break;
 	    case XK_EXE:
-		RunCommand(val.str);
+		RunCommand(val.str.buf);
 		break;
 	    default:
 		abort();
