@@ -96,7 +96,9 @@ static	int	 glob3		__P((Char *, Char *, Char *, Char *,
 				     glob_t *, int));
 static	int	 globextend	__P((Char *, glob_t *));
 static	int	 match		__P((Char *, Char *, Char *, int));
+#ifndef __clipper__
 static	int	 compare	__P((const ptr_t, const ptr_t));
+#endif
 static 	DIR	*Opendir	__P((Char *));
 #ifdef S_IFLNK
 static	int	 Lstat		__P((Char *, struct stat *));
@@ -200,7 +202,7 @@ Stat(fn, sb)
     {
 	int     st;
 
-	st = lstat(buf, sb);
+	st = stat(buf, sb);
 	if (*buf)
 	    dc--;
 	return (*--dc == '/' && !S_ISDIR(sb->st_mode) ? -1 : st);
@@ -245,7 +247,13 @@ static int
 compare(p, q)
     const ptr_t  p, q;
 {
+#if defined(NLS) && !defined(NOSTRCOLL)
+    errno = 0;  /* strcoll sets errno, another brain-damage */
+ 
+    return (strcoll(*(char **) p, *(char **) q));
+#else
     return (strcmp(*(char **) p, *(char **) q));
+#endif /* NLS && !NOSTRCOLL */
 }
 
 /*
@@ -355,7 +363,11 @@ glob(pattern, flags, errfunc, pglob)
 	    break;
 	case STAR:
 	    pglob->gl_flags |= GLOB_MAGCHAR;
-	    *bufnext++ = M_ALL;
+	    /* collapse adjacent stars to one, to avoid
+	     * exponential behavior
+	     */
+	    if (bufnext == patbuf || bufnext[-1] != M_ALL)
+		*bufnext++ = M_ALL;
 	    break;
 	default:
 	    *bufnext++ = CHAR(c);
@@ -448,6 +460,7 @@ glob2(pathbuf, pathend, pattern, pglob, no_match)
     for (;;) {
 	if (*pattern == EOS) {	/* end of pattern? */
 	    *pathend = EOS;
+
 	    if (Lstat(pathbuf, &sbuf))
 		return (0);
 
@@ -500,17 +513,21 @@ glob3(pathbuf, pathend, pattern, restpattern, pglob, no_match)
     struct dirent *dp;
     int     err;
     Char m_not = (pglob->gl_flags & GLOB_ALTNOT) ? M_ALTNOT : M_NOT;
+    char cpathbuf[MAXPATHLEN], *ptr;;
 
     *pathend = EOS;
     errno = 0;
 
-    if (!(dirp = Opendir(pathbuf)))
+    if (!(dirp = Opendir(pathbuf))) {
 	/* todo: don't call for ENOENT or ENOTDIR? */
-	if ((pglob->gl_errfunc && (*pglob->gl_errfunc) (pathbuf, errno)) ||
+	for (ptr = cpathbuf; (*ptr++ = (char) *pathbuf++) != EOS;)
+	    continue;
+	if ((pglob->gl_errfunc && (*pglob->gl_errfunc) (cpathbuf, errno)) ||
 	    (pglob->gl_flags & GLOB_ERR))
 	    return (GLOB_ABEND);
 	else
 	    return (0);
+    }
 
     err = 0;
 
@@ -581,7 +598,7 @@ globextend(path, pglob)
 
     for (p = path; *p++;)
 	continue;
-    if ((copy = (char *) malloc((size_t) (p - path))) != NULL) {
+    if ((copy = (char *) xmalloc((size_t) (p - path))) != NULL) {
 	register char *dc = copy;
 	register Char *sc = path;
 
@@ -612,10 +629,10 @@ match(name, pat, patend, m_not)
 	case M_ALL:
 	    if (pat == patend)
 		return (1);
-	    for (; *name != EOS; ++name) {
+	    do 
 		if (match(name, pat, patend, m_not))
 		    return (1);
-	    }
+	    while (*name++ != EOS);
 	    return (0);
 	case M_ONE:
 	    if (*name++ == EOS)
@@ -640,7 +657,8 @@ match(name, pat, patend, m_not)
 		return (0);
 	    break;
 	default:
-	    if (*name++ != c)
+	    k = *name++;
+	    if (samecase(k) != samecase(c))
 		return (0);
 	    break;
 	}
