@@ -1,4 +1,4 @@
-/* $Header: /u/christos/src/tcsh-6.05/RCS/tc.sched.c,v 3.10 1994/05/26 13:11:20 christos Exp $ */
+/* $Header: /u/christos/src/tcsh-6.06/RCS/tc.sched.c,v 3.13 1995/04/16 19:15:53 christos Exp $ */
 /*
  * tc.sched.c: Scheduled command execution
  *
@@ -38,7 +38,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: tc.sched.c,v 3.10 1994/05/26 13:11:20 christos Exp $")
+RCSID("$Id: tc.sched.c,v 3.13 1995/04/16 19:15:53 christos Exp $")
 
 #include "ed.h"
 #include "tc.h"
@@ -80,7 +80,7 @@ dosched(v, c)
 #ifdef _MINIX
     char kludge[10];
     extern char *sprintf();
-    sprintf(kludge,"kludge");
+    sprintf(kludge, CGETS(24, 1, "kludge"));
 #endif /* _MINIX */
 
     v++;
@@ -91,7 +91,7 @@ dosched(v, c)
 	    fmt = str2short("%h\t%T\t%R\n");
 	/* print list of scheduled events */
 	for (count = 1, tp = sched_ptr; tp; count++, tp = tp->t_next) {
-	    Char buf[BUFSIZE], sbuf[BUFSIZE], *cp;
+	    Char buf[BUFSIZE], sbuf[BUFSIZE];
 	    blkexpand(tp->t_lex, buf);
 	    tprintf(FMT_SCHED, sbuf, fmt, sizeof(sbuf), 
 		    short2str(buf), tp->t_when, (ptr_t) &count);
